package com.extensis.web.common.JMS;

import java.util.Map;

import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Session;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicSession;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import com.extensis.web.common.WebFontJMSException;

/*
 * This class is used to publish one or more messages to a JMS topic. The static sendMessage members are used to send a single message
 * to a topic. The non-static members provide improved performance when multiple messages need to be sent to the same topic. 
 */
public class TopicPublisher {
	/*
	 * Publish a single message to a given topic using the default JMS connection factory.
	 */
	/*
	 * Publish a single message to a given topic using a given connection factory.
	 */
	public static void sendMessage(String topicName, String connectionFactoryName, Map<String, Object> msgContents) throws WebFontJMSException {
		if (null==topicName || topicName.length()==0) {
			throw new IllegalArgumentException("topicName cannot be null or empty");
		}
		if (null==connectionFactoryName || connectionFactoryName.length()==0) {
			throw new IllegalArgumentException("connectionFactoryName cannot be null or empty");
		}
		if (null==msgContents || msgContents.size()==0) {
			throw new IllegalArgumentException("msgContent cannot be null or empty");
		}

		try {
			Context initialContext = new InitialContext();
	
			try {
				String topicJNDIName = Topics.TOPIC_JNDI_PREFIX + topicName;
				Topic topic = (Topic) initialContext.lookup(topicJNDIName);
				
				TopicConnectionFactory topicConnectionFactory = (TopicConnectionFactory) initialContext.lookup(connectionFactoryName);
	
				TopicConnection topicConnection = topicConnectionFactory.createTopicConnection();
	
				try {
					TopicSession topicSession = topicConnection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
	
					javax.jms.TopicPublisher topicPublisher = topicSession.createPublisher(topic);
					
					sendMessage(topicSession, topicPublisher, msgContents);
				} finally {
					topicConnection.close();
				}
			}
			finally {
				initialContext.close();
			}
		}
		catch (NamingException ne) {
			throw new WebFontJMSException(ne);
		}
		catch (JMSException jmse) {
			throw new WebFontJMSException(jmse);
		}
	}
	
	/*
	 * This private static method is used internally to publish a message. It is used by both the static and non-static sendMessage methods.
	 */
	private static void sendMessage(TopicSession topicSession, javax.jms.TopicPublisher topicPublisher, Map<String, Object> msgContents) throws WebFontJMSException {
		try {
			MapMessage mapMsg = topicSession.createMapMessage();
			for (String key : msgContents.keySet()) {
				Object value = msgContents.get(key);
				mapMsg.setObject(key, value);
			}
			topicPublisher.publish(mapMsg);
		}
		catch (JMSException jmse) {
			throw new WebFontJMSException(jmse);
		}
	}

	private TopicConnection topicConnection = null;
	private TopicSession topicSession = null;
	private javax.jms.TopicPublisher topicPublisher = null;
	
	/*
	 * Prepare to publish multiple messages to the same topic using a given connection factory. 
	 */
	public TopicPublisher(String topicName, String connectionFactoryName) throws WebFontJMSException {
		if (null==topicName || topicName.length()==0) {
			throw new IllegalArgumentException("topicName cannot be null or empty");
		}
		if (null==connectionFactoryName || connectionFactoryName.length()==0) {
			throw new IllegalArgumentException("connectionFactoryName cannot be null or empty");
		}

		try {
			Context initialContext = new InitialContext();
		
			try {
				String topicJNDIName = Topics.TOPIC_JNDI_PREFIX + topicName;
				Topic topic = (Topic) initialContext.lookup(topicJNDIName);
				
				TopicConnectionFactory topicConnectionFactory = (TopicConnectionFactory) initialContext.lookup(connectionFactoryName);
		
				topicConnection = topicConnectionFactory.createTopicConnection();
		
				boolean success = false;
				try {
					topicSession = topicConnection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
		
					topicPublisher = topicSession.createPublisher(topic);
					
					success = true;
				} finally {
					if (!success) {
						topicConnection.close();
					}
				}
			}
			finally {
				initialContext.close();
			}
		}
		catch (NamingException ne) {
			throw new WebFontJMSException(ne);
		}
		catch (JMSException jmse) {
			throw new WebFontJMSException(jmse);
		}
	}
	
	/*
	 * Publish one of multiple messages.
	 */
	public void sendMessage(Map<String, Object> msgContents) throws WebFontJMSException {
		if (null==msgContents || msgContents.size()==0) {
			throw new IllegalArgumentException("msgContent cannot be empty");
		}
		
		sendMessage(topicSession, topicPublisher, msgContents);
	}
	
	/*
	 * Must be called after publishing multiple messages.
	 */
	public void close() throws WebFontJMSException {
		try {
			topicConnection.close();
		}
		catch (JMSException jmse) {
			throw new WebFontJMSException(jmse);
		}
	}
}
